# Copyright (C) 2016, A10 Networks Inc. All rights reserved.

#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from __future__ import absolute_import, unicode_literals

from acos_client import client

try:
    import unittest2 as unittest
except ImportError:
    import unittest


class TestClient(unittest.TestCase):

    def setUp(self):
        self.client_21 = client.Client('fake-host', '2.1', 'fake-username', 'fake-password', max_retries=5, timeout=3)
        self.client_30 = client.Client('fake-host', '3.0', 'fake-username', 'fake-password', max_retries=6, timeout=4)

    def test_dns_v21(self):
        from acos_client.v21.dns import DNS

        self.assertIsInstance(self.client_21.dns, DNS)

    def test_dns_v30(self):
        from acos_client.v30.dns import DNS

        self.assertIsInstance(self.client_30.dns, DNS)

    def test_max_retries_v21(self):

        self.assertEqual(self.client_21.max_retries, 5)
        self.assertEqual(self.client_21.timeout, 3)
        self.assertEqual(self.client_21.http.max_retries, 5)
        self.assertEqual(self.client_21.http.timeout, 3)

    def test_max_retries_v30(self):

        self.assertEqual(self.client_30.max_retries, 6)
        self.assertEqual(self.client_30.timeout, 4)
        self.assertEqual(self.client_30.http.max_retries, 6)
        self.assertEqual(self.client_30.http.timeout, 4)



# Copyright (C) 2021, A10 Networks Inc. All rights reserved.

#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import
from __future__ import unicode_literals

from acos_client.v30 import base
from acos_client.v30.delete import glm_license


class Delete(base.BaseV30):

    @property
    def glm_license(self):
        return glm_license.DeleteGLMLicense(self.client)



# Copyright (C) 2016, A10 Networks Inc. All rights reserved.

#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import logging
from requests.adapters import HTTPAdapter
import ssl

FORCED_CIPHERS = (
    'ECDH+AESGCM:DH+AESGCM:ECDH+AES256:DH+AES256:ECDH+AES128:DH+AES:ECDH+HIGH:'
    'DH+HIGH:ECDH+3DES:DH+3DES:RSA+AESGCM:RSA+AES:RSA+HIGH:RSA+3DES'
)


class SSLAdapter(HTTPAdapter):
    """A TransportAdapter that re-enables 3DES support in Requests.

    """

    def create_ssl_context(self):
        ctx = ssl.create_default_context()
        # Disable all encryption protcols except TLS1_0
        ctx.options |= ssl.OP_NO_SSLv2 | ssl.OP_NO_SSLv3
        # Try-Except here because OP_NO_TLSv1_3 not available in Python3 before 3.6
        try:
            ctx.options |= ssl.OP_NO_TLSv1_3 | ssl.OP_NO_TLSv1_2 | ssl.OP_NO_TLSv1_1
        except(AttributeError):
            ctx.options |= ssl.OP_NO_TLSv1_2 | ssl.OP_NO_TLSv1_1
        ctx.set_ciphers(FORCED_CIPHERS)
        ctx.check_hostname = False
        return ctx

    def init_poolmanager(self, *args, **kwargs):
        logging.debug(' ----------- SSLAdapter.init_poolmanager -------------- ')
        kwargs['ssl_context'] = self.create_ssl_context()
        return super(SSLAdapter, self).init_poolmanager(*args, **kwargs)

    def proxy_manager_for(self, *args, **kwargs):
        logging.debug(' ----------- SSLAdapter.proxy_manager_for -------------- ')
        kwargs['ssl_context'] = self.create_ssl_context()
        return super(SSLAdapter, self).proxy_manager_for(*args, **kwargs)



# Copyright 2014,  Jeff Buttars,  A10 Networks.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import
from __future__ import unicode_literals

from acos_client.v30 import base
from acos_client.v30.slb.member import Member


class ServiceGroup(base.BaseV30):
    url_prefix = '/slb/service-group/'

    @property
    def member(self):
        return Member(self.client)

    # Valid LB methods
    ROUND_ROBIN = 'round-robin'
    WEIGHTED_ROUND_ROBIN = 'weighted-rr'
    LEAST_CONNECTION = 'least-connection'
    WEIGHTED_LEAST_CONNECTION = 'weighted-least-connection'
    LEAST_CONNECTION_ON_SERVICE_PORT = 'service-least-connection'
    WEIGHTED_LEAST_CONNECTION_ON_SERVICE_PORT = \
        'service-weighted-least-connection'
    FAST_RESPONSE_TIME = 'fastest-response'
    LEAST_REQUEST = 'least-request'
    STRICT_ROUND_ROBIN = 'round-robin-strict'
    STATELESS_SOURCE_IP_HASH = 'stateless-src-ip-hash'
    STATELESS_SOURCE_IP_HASH_ONLY = 'stateless-src-ip-only-hash'
    STATELESS_DESTINATION_IP_HASH = 'stateless-dst-ip-hash'
    STATELESS_SOURCE_DESTINATION_IP_HASH = 'stateless-src-dst-ip-hash'
    STATELESS_PER_PACKET_ROUND_ROBIN = 'stateless-per-pkt-round-robin'
    SOURCE_IP_HASH_ONLY = 'src-ip-only-hash'
    SOURCE_IP_HASH = 'src-ip-hash'
    DESTINATION_IP_HASH_ONLY = 'dst-ip-only-hash'
    DESTINATION_IP_HASH = 'dst-ip-hash'

    # Valid protocols
    TCP = 'tcp'
    UDP = 'udp'

    def _set(self, name, protocol=None, lb_method=None, service_group_templates=None,
             hm_name=None, mem_list=None, health_check_disable=False, health_check=None,
             hm_delete=False, **kwargs):

        # Normalize "" -> None for json
        hm_name = hm_name or None

        # v30 needs unit tests badly...

        params = {
            "service-group": self.minimal_dict({
                "name": name,
                "protocol": protocol,
                "member-list": mem_list
            })
        }

        # If we explicitly disable health checks, ensure it happens
        # Else, we implicitly disable health checks if not specified.
        health_check_disable = 1 if health_check_disable else 0

        # When enabling/disabling a health monitor, you can't specify
        # health-check-disable and health-check at the same time.
        if hm_name is None:
            params["service-group"]["health-check-disable"] = health_check_disable
            # Have to explicitly detach health monitor from the service group,
            # by setting hm_delete flag
            if hm_delete:
                params["service-group"]["health-check"] = health_check
        else:
            params["service-group"]["health-check"] = hm_name

        if lb_method is None:
            pass
        elif lb_method[-16:] == 'least-connection':
            params['service-group']['lc-method'] = lb_method
            params['service-group']['stateless-auto-switch'] = 0
        elif lb_method[:9] == 'stateless':
            params['service-group']['stateless-lb-method'] = lb_method
        else:
            params['service-group']['lb-method'] = lb_method
            params['service-group']['stateless-auto-switch'] = 0

        if service_group_templates:
            service_group_templates = {k: v for k, v in service_group_templates.items() if v}

            if service_group_templates.get('template-policy'):
                params['service-group']['template-policy'] = service_group_templates['template-policy']
            elif service_group_templates.get('template-policy-shared'):
                params['service-group']['template-policy-shared'] = service_group_templates['template-policy-shared']
                params['service-group']['shared-partition-policy-template'] = True

            params['service-group']['template-server'] = service_group_templates.get('template-server', None)
            params['service-group']['template-port'] = service_group_templates.get('template-port', None)

        return params

    def all(self, max_retries=None, timeout=None, *args, **kwargs):
        return self._get(self.url_prefix, max_retries=max_retries, timeout=timeout, axapi_args=kwargs)

    def all_stats(self, max_retries=None, timeout=None, *args, **kwargs):
        return self._get(self.url_prefix + "stats", max_retries=max_retries, timeout=timeout, axapi_args=kwargs)

    def all_oper(self, max_retries=None, timeout=None, *args, **kwargs):
        return self._get(self.url_prefix + "oper", max_retries=max_retries, timeout=timeout, axapi_args=kwargs)

    def create(self, name, protocol=TCP, lb_method=ROUND_ROBIN, service_group_templates=None,
               mem_list=None, hm_name=None, max_retries=None, timeout=None, health_check_disable=False,
               health_check=None, **kwargs):
        params = self._set(name, protocol=protocol, lb_method=lb_method,
                           service_group_templates=service_group_templates,
                           mem_list=mem_list, hm_name=hm_name, health_check_disable=health_check_disable,
                           health_check=health_check, **kwargs)
        return self._post(self.url_prefix, params, max_retries=max_retries, timeout=timeout, axapi_args=kwargs)

    def delete(self, name):
        return self._delete(self.url_prefix + name)

    def get(self, name, max_retries=None, timeout=None, **kwargs):
        return self._get(self.url_prefix + name, max_retries=max_retries, timeout=timeout, axapi_args=kwargs)

    def oper(self, name, max_retries=None, timeout=None, *args, **kwargs):
        return self._get(self.url_prefix + name + "/oper", max_retries=max_retries, timeout=timeout,
                         axapi_args=kwargs)

    def stats(self, name, max_retries=None, timeout=None, *args, **kwargs):
        return self._get(self.url_prefix + name + "/stats", max_retries=max_retries, timeout=timeout,
                         axapi_args=kwargs)

    def update(self, name, protocol=None, lb_method=None, health_monitor=None,
               service_group_templates=None, mem_list=None, hm_name=None,
               max_retries=None, timeout=None, health_check_disable=False, health_check=None,
               hm_delete=False, **kwargs):
        params = self._set(name, protocol=None, lb_method=lb_method, hm_name=hm_name,
                           service_group_templates=service_group_templates,
                           mem_list=mem_list, health_check_disable=health_check_disable,
                           health_check=health_check, hm_delete=hm_delete, **kwargs)
        return self._post(self.url_prefix + name, params, max_retries=max_retries, timeout=timeout,
                          axapi_args=kwargs)

    def replace(self, name, protocol=None, lb_method=None, health_monitor=None,
                service_group_templates=None, mem_list=None, hm_name=None,
                max_retries=None, timeout=None, health_check_disable=False, health_check=None,
                **kwargs):
        params = self._set(name, protocol=protocol, lb_method=lb_method, hm_name=hm_name,
                           service_group_templates=service_group_templates,
                           mem_list=mem_list, health_check_disable=health_check_disable,
                           health_check=health_check, **kwargs)
        return self._put(self.url_prefix + name, params, max_retries=max_retries, timeout=timeout,
                         axapi_args=kwargs)



# Copyright 2018,  A10 Networks.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os

try:
    import unittest2 as unittest
except ImportError:
    import unittest


def test_suite():
    os.environ.setdefault("PYTHONDONTWRITEBYTECODE", "1")
    test_loader = unittest.TestLoader()
    test_suite = test_loader.discover("acos_client.tests")
    return test_suite



# Copyright 2014,  Doug Wiegley,  A10 Networks.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


class RequiredAttributeNotSpecified(Exception):

    def __init__(self, url, attribute, requires=[]):
        required_str = str(requires).strip("[]")
        self.message = ("The attribute {attribute} requires that {requires}"
                        "also be specified when querying "
                        "the {url} endpoint.").format(url, attribute, required_str)
        super(RequiredAttributeNotSpecified, self).__init__(self.message)


class ACOSException(Exception):
    def __init__(self, code=1, msg=''):
        self.code = code
        self.msg = msg
        super(ACOSException, self).__init__(msg)

    def __str__(self):
        return "%d %s" % (self.code, self.msg)


class ACOSUnsupportedVersion(ACOSException):
    pass


class ACOSUnknownError(ACOSException):
    pass


class AddressSpecifiedIsInUse(ACOSException):
    pass


class AuthenticationFailure(ACOSException):
    pass


class InvalidSessionID(ACOSException):
    pass


class Exists(ACOSException):
    pass


class NotFound(ACOSException):
    pass


class NoSuchServiceGroup(ACOSException):
    pass


class NotImplemented(ACOSException):
    pass


class InUse(ACOSException):
    pass


class InvalidPartitionParameter(ACOSException):
    pass


class MemoryFault(ACOSException):
    pass


class InvalidParameter(ACOSException):
    pass


class OutOfPartitions(ACOSException):
    pass


class PartitionIdExists(ACOSException):
    pass


class HMMissingHttpPassive(ACOSException):
    pass


class AxapiJsonFormatError(ACOSException):
    pass


class ConfigManagerNotReady(ACOSException):
    pass


class DhcpAcquireFailed(ACOSException):
    pass


class InvalidInteger(ACOSException):
    pass


class CertificateParsingFailed(ACOSException):
    pass


class KeyParsingFailed(ACOSException):
    pass


class FeatureNotSupported(ACOSException):
    pass


class ACOSSystemNotReady(ACOSException):
    pass


class ACOSSystemIsBusy(ACOSException):
    pass


class LicenseOptionNotAllowed(ACOSException):
    pass



# Copyright 2014,  Doug Wiegley,  A10 Networks.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import
from __future__ import unicode_literals

from acos_client.v21 import base
from acos_client.v21.slb.member import Member


class ServiceGroup(base.BaseV21):

    @property
    def member(self):
        return Member(self.client)

    # Valid LB methods
    ROUND_ROBIN = 0
    WEIGHTED_ROUND_ROBIN = 1
    LEAST_CONNECTION = 2
    WEIGHTED_LEAST_CONNECTION = 3
    LEAST_CONNECTION_ON_SERVICE_PORT = 4
    WEIGHTED_LEAST_CONNECTION_ON_SERVICE_PORT = 5
    FAST_RESPONSE_TIME = 6
    LEAST_REQUEST = 7
    STRICT_ROUND_ROBIN = 8
    STATELESS_SOURCE_IP_HASH = 9
    STATELESS_SOURCE_IP_HASH_ONLY = 10
    STATELESS_DESTINATION_IP_HASH = 11
    STATELESS_SOURCE_DESTINATION_IP_HASH = 12
    STATELESS_PER_PACKET_ROUND_ROBIN = 13
    SOURCE_IP_HASH_ONLY = 14
    SOURCE_IP_HASH = 15
    DESTINATION_IP_HASH_ONLY = 16
    DESTINATION_IP_HASH = 17

    # Valid protocols
    TCP = 2
    UDP = 3

    def _set(self, action, name, protocol=None, lb_method=None, hm_name=None,
             **kwargs):
        params = {
            "service_group": self.minimal_dict({
                "name": name,
                "protocol": protocol,
                "lb_method": lb_method,
                "health_monitor": hm_name
            })
        }
        return self._post(action, params, **kwargs)

    def all(self, **kwargs):
        return self._get('slb.service_group.getAll', **kwargs)

    def all_delete(self, **kwargs):
        return self._get('slb.service_group.deleteAll', **kwargs)

    def all_stats(self, **kwargs):
        return self._get("slb.service_group.fetchAllStatistics", **kwargs)

    def create(self, name, protocol=TCP, lb_method=ROUND_ROBIN, **kwargs):
        return self._set("slb.service_group.create", name, protocol, lb_method, **kwargs)

    def delete(self, name, **kwargs):
        return self._post("slb.service_group.delete", {'name': name}, **kwargs)

    def get(self, name, **kwargs):
        return self._post("slb.service_group.search", {'name': name}, **kwargs)

    def stats(self, name, **kwargs):
        return self._post("slb.service_group.fetchStatistics", {"name": name}, **kwargs)

    def update(self, name, protocol=None, lb_method=None, health_monitor=None, **kwargs):
        return self._set(
            "slb.service_group.update", name, protocol, lb_method, health_monitor, **kwargs
        )



# Copyright 2016,  A10 Networks, All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import
from __future__ import unicode_literals

from acos_client import errors as acos_errors
from acos_client.v30 import base


class SFlow(base.BaseV30):

    url_prefix = '/sflow/'

    @property
    def collector(self):
        return SFlowCollector(self.client)

    @property
    def setting(self):
        return SFlowSetting(self.client)

    @property
    def polling(self):
        return SFlowPolling(self.client)


class SFlowSetting(base.BaseV30):
    url_prefix = "/sflow/setting"

    def create(self, max_header, source_ip_use_mgmt,
               packet_sampling_rate, counter_polling_interval, **kwargs):
        params = {
            "setting": {
                # "max-header": max_header,
                # "source-ip-use-mgmt": int(source_ip_use_mgmt),
                # "packet-sampling-rate": packet_sampling_rate,
                "counter-polling-interval": counter_polling_interval
            }
        }

        return self._post(self.url_prefix, params, **kwargs)

    def get(self):
        return self._get(self.url_prefix)


class SFlowCollectorIP(base.BaseV30):
    url_prefix = "/sflow/collector/ip"

    def create(self, ip_address, port, **kwargs):
        params = {
            "ip":
                [{
                    "addr": ip_address,
                    "port": int(port)
                }]

        }

        return self._post(self.url_prefix, params, **kwargs)

    def get(self, ip_address, port, **kwargs):
        url = "{0}/{1}+{2}".format(self.url_prefix, ip_address, port)
        return self._get(url, **kwargs)


class SFlowCollector(base.BaseV30):
    @property
    def ip(self):
        return SFlowCollectorIP(self.client)


class SFlowPolling(base.BaseV30):
    url_prefix = "/sflow/polling"

    def create(self, http_counter=False, **kwargs):
        params = {
            "polling": {
                "http-counter": http_counter
            }
        }
        try:
            return self._post(self.url_prefix, params, **kwargs)
        except acos_errors.Exists:
            pass

    def get(self, **kwargs):
        return self._get(self.url_prefix, **kwargs)



# Copyright 2014,  Doug Wiegley,  A10 Networks.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from __future__ import absolute_import
from __future__ import unicode_literals

import json
import logging
import time

from requests.adapters import HTTPAdapter
from requests import Session

import acos_client
from acos_client import logutils
from acos_client.v30 import responses as acos_responses

LOG = logging.getLogger(__name__)

broken_replies = {
    "": '{"response": {"status": "OK"}}'
}


class HttpClient(object):
    AXAPI_DEFAULT_REQ_TIMEOUT = 300
    HEADERS = {
        "Content-type": "application/json",
        "User-Agent": "ACOS-Client-AGENT-%s" % acos_client.VERSION,
    }

    def __init__(self, host, port=None, protocol="https", max_retries=3,
                 timeout=AXAPI_DEFAULT_REQ_TIMEOUT):
        if port is None:
            if protocol == 'http':
                self.port = 80
            else:
                self.port = 443
        else:
            self.port = port

        self.url_base = "%s://%s:%s" % (protocol, host, self.port)
        self.max_retries = max_retries
        self.timeout = timeout

    def dict_underscore_to_dash(self, my_dict):
        if type(my_dict) is list:
            item_list = []
            for item in my_dict:
                item_list.append(self.dict_underscore_to_dash(item))
            return item_list
        elif type(my_dict) is dict:
            item_dict = {}
            for k, v in my_dict.items():
                item_dict[k.replace('_', '-')] = self.dict_underscore_to_dash(v)
            return item_dict
        else:
            return my_dict

    def request_impl(self, method, api_url, params={}, headers=None,
                     file_name=None, file_content=None, axapi_args=None,
                     max_retries=None, timeout=None, **kwargs):
        LOG.debug("axapi_http: full url = %s", self.url_base + api_url)
        LOG.debug("axapi_http: %s url = %s", method, api_url)
        LOG.debug("axapi_http: params = %s", json.dumps(logutils.clean(params), indent=4))

        valid_http_codes = [200, 204]

        # Update params with axapi_args for currently unsupported configuration of objects
        if axapi_args is not None:
            formatted_axapi_args = self.dict_underscore_to_dash(axapi_args)
            params = acos_client.v21.axapi_http.merge_dicts(params, formatted_axapi_args)

        LOG.debug("axapi_http: params + axapi_args = %s", json.dumps(logutils.clean(params), indent=4))
        # Set data" variable for the request
        if params:
            params_copy = params.copy()
            LOG.debug("axapi_http: params_all = %s", logutils.clean(params_copy))
            payload = json.dumps(params_copy)
        else:
            payload = None

        if (file_name is None and file_content is not None) or \
           (file_name is not None and file_content is None):
            raise ValueError("file_name and file_content must both be populated if one is")

        if not max_retries:
            max_retries = self.max_retries
        if not timeout:
            timeout = self.timeout

        # Set "headers" variable for the request
        request_headers = self.HEADERS.copy()
        if headers:
            request_headers.update(headers)
        LOG.debug("axapi_http: headers = %s", json.dumps(logutils.clean(request_headers), indent=4))

        # Process files if passed as a parameter
        if file_name is not None:
            files = {
                'file': (file_name, file_content, "application/octet-stream"),
                'json': ('blob', payload, "application/json")
            }
            request_headers.pop("Content-type", None)
            request_headers.pop("Content-Type", None)

        # Create session to set HTTPAdapter or SSLAdapter and set max_retries
        session = Session()
        if self.port == 443:
            session.mount('https://', HTTPAdapter(max_retries=max_retries))
        else:
            session.mount('http://', HTTPAdapter(max_retries=max_retries))
        session_request = getattr(session, method.lower())

        # Make actual request and handle any errors
        try:
            if file_name is not None:
                device_response = session_request(
                    self.url_base + api_url, verify=False, files=files, headers=request_headers, timeout=timeout
                )
            else:
                device_response = session_request(
                    self.url_base + api_url, verify=False, data=payload, headers=request_headers, timeout=timeout
                )
        except (Exception) as e:
            LOG.error("acos_client failing with error %s after %s retries", e.__class__.__name__, max_retries)
            raise e
        finally:
            session.close()

        # Validate json response
        try:
            json_response = device_response.json()
            LOG.debug("axapi_http: data = %s", json.dumps(logutils.clean(json_response), indent=4))
        except ValueError as e:
            # The response is not JSON but it still succeeded.
            if device_response.status_code in valid_http_codes:
                return device_response.text
            else:
                raise e

        # Handle "fail" responses returned by AXAPI
        if 'response' in json_response and 'status' in json_response['response']:
            if json_response['response']['status'] == 'fail':
                    acos_responses.raise_axapi_ex(json_response, method, api_url)

        # Handle "authorizationschema" responses returned by AXAPI
        if 'authorizationschema' in json_response:
            acos_responses.raise_axapi_auth_error(json_response, method, api_url, headers)

        return json_response

    def request(self, method, api_url, params={}, headers=None,
                file_name=None, file_content=None, axapi_args=None,
                max_retries=None, timeout=None, **kwargs):
        retry_timeout = 300
        if timeout and timeout > retry_timeout:
            retry_timeout = timeout
        start_time = time.time()
        loop = True

        while loop:
            try:
                return self.request_impl(method, api_url, params, headers,
                                         file_name=file_name, file_content=file_content,
                                         max_retries=max_retries,
                                         timeout=timeout, axapi_args=axapi_args,
                                         **kwargs)

            except acos_responses.axapi_retry_exceptions() as e:
                LOG.warning("ACOS device system is busy: %s", str(e))
                loop = ((time.time() - start_time) <= retry_timeout)
                if not loop:
                    raise e
                time.sleep(1)
            except (Exception) as e:
                raise e

    def get(self, api_url, params={}, headers=None, max_retries=None, timeout=None, axapi_args=None, **kwargs):
        return self.request("GET", api_url, params, headers, max_retries=max_retries,
                            timeout=timeout, axapi_args=axapi_args, **kwargs)

    def post(self, api_url, params={}, headers=None, max_retries=None, timeout=None, axapi_args=None, **kwargs):
        return self.request("POST", api_url, params, headers, max_retries=max_retries,
                            timeout=timeout, axapi_args=axapi_args, **kwargs)

    def put(self, api_url, params={}, headers=None, max_retries=None, timeout=None, axapi_args=None, **kwargs):
        return self.request("PUT", api_url, params, headers, max_retries=max_retries,
                            timeout=timeout, axapi_args=axapi_args, **kwargs)

    def delete(self, api_url, params={}, headers=None, max_retries=None, timeout=None, axapi_args=None, **kwargs):
        return self.request("DELETE", api_url, params, headers, max_retries=max_retries,
                            timeout=timeout, axapi_args=axapi_args, **kwargs)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import
from __future__ import unicode_literals

from acos_client.v30 import base


class Port(base.BaseV30):

    url_base_tmpl = "/slb/server/{server}/port/"
    url_port_tmpl = "{port}+{protocol}/"

    def create(self, server_name, port, protocol, max_retries=None, timeout=None,
               conn_resume=None, conn_limit=None, stats_data_action="stats-data-enable",
               weight=1, range=0, action="enable", **kwargs):
        url = self.url_base_tmpl.format(server=server_name)
        params = self._set(server_name, port, protocol, conn_resume=conn_resume, conn_limit=conn_limit,
                           stats_data_action=stats_data_action, weight=weight, range=range, action=action)
        return self._post(url, params, max_retries=max_retries, timeout=timeout, axapi_args=kwargs)

    def update(self, server_name, port, protocol, max_retries=None, timeout=None,
               conn_resume=None, conn_limit=None, stats_data_action="stats-data-enable",
               weight=1, range=0, action="enable", **kwargs):
        url = self.url_base_tmpl.format(server=server_name)
        url += self.url_port_tmpl.format(port=port, protocol=protocol)
        params = self._set(server_name, port, protocol, conn_resume=conn_resume, conn_limit=conn_limit,
                           stats_data_action=stats_data_action, weight=weight, range=range, action=action)
        return self._put(url, params, max_retries=max_retries, timeout=timeout, axapi_args=kwargs)

    def delete(self, server_name, port, protocol, **kwargs):
        url = (self.url_base_tmpl + self.url_port_tmpl).format(server=server_name,
                                                               port=port,
                                                               protocol=protocol)

        return self._delete(url)

    def _set(self, server_name, port, protocol, conn_resume=None, conn_limit=None,
             stats_data_action="stats-data-enable", weight=1, range=0, action="enable"):
        params = {
            "port": {
                "stats-data-action": stats_data_action,
                "weight": weight,
                "port-number": port,
                "range": range,
                "action": action,
                "protocol": protocol
            }
        }

        if conn_resume:
            params['port']['conn-resume'] = conn_resume

        if conn_limit:
            params['port']['conn-limit'] = conn_limit

        return params



# Copyright 2014,  Doug Wiegley,  A10 Networks.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import
from __future__ import unicode_literals

from acos_client import errors as acos_errors
from acos_client.v21 import base


class VirtualPort(base.BaseV21):

    # Protocols
    TCP = 2
    UDP = 3
    HTTP = 11
    HTTPS = 12
    OTHERS = 4
    RTSP = 5
    FTP = 6
    MMS = 7
    SIP = 8
    FAST_HTTP = 9
    GENERIC_PROXY = 10
    SSL_PROXY = 13
    SMTP = 14
    SIP_TCP = 15
    SIPS = 16
    DIAMETER = 17
    DNS_UDP = 18
    TFTP = 19
    DNS_TCP = 20
    RADIUS = 21
    MYSQL = 22
    MSSQL = 23
    FIX = 24
    SMPP_TCP = 25
    SPDY = 26
    SPDYS = 27
    FTP_PROXY = 28

    # The keys as specified in the ACOS JSON message.
    CLIENT_SSL_TMPL_KEY = "client_ssl_template"
    SERVER_SSL_TMPL_KEY = "server_ssl_template"

    # The keys as sent from a10-neutron-lbaas
    # They match what we use in v4 so we transform here
    CLIENT_SSL_ANL_KEY = "template_client_ssl"
    SERVER_SSL_ANL_KEY = "template_server_ssl"

    def _set(
        self,
        action,
        virtual_server_name,
        name,
        protocol,
        port,
        service_group_name,
        s_pers_name=None,
        c_pers_name=None,
        status=1,
        autosnat=False,
        ipinip=False,
        source_nat=None,
        ha_conn_mirror=None,
        no_dest_nat=None,
        conn_limit=None,
        tcp_template=None,
        udp_template=None,
        **kwargs
    ):
        params = {
            "name": virtual_server_name,
            "vport": self.minimal_dict({
                "name": name,
                "service_group": service_group_name,
                "protocol": int(protocol),
                "port": int(port),
                "source_ip_persistence_template": s_pers_name,
                "cookie_persistence_template": c_pers_name,
                "status": status
            })
        }
        client_ssl_template = kwargs.get(self.CLIENT_SSL_TMPL_KEY)
        server_ssl_template = kwargs.get(self.SERVER_SSL_TMPL_KEY)

        if client_ssl_template:
            params['vport'][self.CLIENT_SSL_ANL_KEY] = client_ssl_template

        if server_ssl_template:
            params['vport'][self.SERVER_SSL_ANL_KEY] = server_ssl_template

        if autosnat:
            params['vport']['source_nat_auto'] = int(autosnat)
        if ipinip:
            params['vport']['ip_in_ip'] = int(ipinip)
        if source_nat and len(source_nat) > 0:
            params['vport']['source_nat'] = source_nat
        if tcp_template:
            params['vport']['tcp_template'] = tcp_template
        if udp_template:
            params['vport']['udp_template'] = udp_template
        if no_dest_nat is not None:
            params['vport']['no-dest-nat'] = 1 if no_dest_nat else 0
        if ha_conn_mirror is not None:
            params['vport']['ha-conn-mirror'] = 1 if ha_conn_mirror else 0
        if conn_limit is not None:
            conn_limit = int(conn_limit)
            if conn_limit > 0 and conn_limit <= 8000000:
                params['vport']['conn-limit'] = conn_limit

        return self._post(action, params, **kwargs)

    def get(self, virtual_server_name, name, protocol, port, **kwargs):
        # There is no slb.virtual_server.vport.search.
        # Instead, we get the virtual server and get the desired vport.
        results = self._post('slb.virtual_server.search', {'name': virtual_server_name}, **kwargs)

        vport_list = results.get("virtual_server", []).get("vport_list", [])
        vport = None
        for vport_dict in vport_list:
            if (
                vport_dict.get("name", None) == name and
                vport_dict.get("protocol", None) == int(protocol) and
                vport_dict.get("port", None) == int(port)
            ):
                vport = vport_dict
        if vport is not None:
            return vport
        else:
            raise acos_errors.NotFound()

    def create(
        self,
        virtual_server_name,
        name,
        protocol,
        port,
        service_group_name,
        s_pers_name=None,
        c_pers_name=None,
        status=1,
        autosnat=False,
        ipinip=False,
        source_nat_pool=None,
        ha_conn_mirror=None,
        no_dest_nat=None,
        conn_limit=None,
        tcp_template=None,
        udp_template=None,
        **kwargs
    ):
        return self._set(
            'slb.virtual_server.vport.create',
            virtual_server_name,
            name,
            protocol,
            port,
            service_group_name,
            s_pers_name,
            c_pers_name,
            status,
            autosnat=autosnat,
            ipinip=ipinip,
            source_nat=source_nat_pool,
            ha_conn_mirror=ha_conn_mirror,
            no_dest_nat=no_dest_nat,
            conn_limit=conn_limit,
            tcp_template=tcp_template,
            udp_template=udp_template,
            **kwargs
        )

    def update(
        self,
        virtual_server_name,
        name,
        protocol,
        port,
        service_group_name,
        s_pers_name=None,
        c_pers_name=None,
        status=1,
        autosnat=False,
        ipinip=False,
        source_nat_pool=None,
        ha_conn_mirror=None,
        no_dest_nat=None,
        conn_limit=None,
        tcp_template=None,
        udp_template=None,
        **kwargs
    ):
            return self._set(
                'slb.virtual_server.vport.update',
                virtual_server_name,
                name,
                protocol,
                port,
                service_group_name,
                s_pers_name,
                c_pers_name,
                status,
                autosnat=autosnat,
                ipinip=ipinip,
                source_nat=source_nat_pool,
                ha_conn_mirror=ha_conn_mirror,
                no_dest_nat=no_dest_nat,
                conn_limit=conn_limit,
                tcp_template=tcp_template,
                udp_template=udp_template,
                **kwargs
            )

    def delete(self, virtual_server_name, name, protocol, port, **kwargs):
        params = {
            "name": virtual_server_name,
            "vport": {
                "name": name,
                "protocol": protocol,
                "port": int(port)
            }
        }
        return self._post("slb.virtual_server.vport.delete", params, **kwargs)



# Copyright 2015, A10 Networks
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import
from __future__ import unicode_literals

from acos_client import errors as acos_errors
from acos_client.v30 import base


class RIB(base.BaseV30):
    """Manipulation of Routing Information Base"""
    url_prefix = "/ip/route/rib"

    def create(self, destination, mask, next_hops=[]):
        """Create route to {destination} {mask} using {next_hops} expressed as (gateway, distance)"""
        payload = {
            "rib": self._build_payload(destination, mask, next_hops)
        }

        return self._post(self.url_prefix, payload)

    def get(self, destination, mask):
        return self._get(self._build_url(destination, mask))

    def exists(self, destination, mask):
        try:
            self.get(destination, mask)
            return True
        except acos_errors.NotFound:
            return False

    def delete(self, destination, mask):
        return self._delete(self._build_url(destination, mask))

    def update(self, destination, mask, next_hops=[]):
        payload = {
            "rib": self._build_payload(destination, mask, next_hops)
        }
        return self._put(self._build_url(destination, mask), payload)

    def get_all(self):
        return self._get(self.url_prefix)

    def _build_nexthops(self, nexthops):
        hops = []
        for ip, dist in nexthops:
            hops.append({"ip-next-hop": ip, "distance-nexthop-ip": dist})
        return hops

    def _build_payload(self, destination, mask, next_hops):
        hop_array = self._build_nexthops(next_hops)

        payload = {
            "ip-dest-addr": destination,
            "ip-mask": mask
        }

        if len(hop_array) > 0:
            payload["ip-nexthop-ipv4"] = hop_array

        return payload

    def _build_url(self, destination, mask):
        # Self explanatory except the mask. We're trimming the leading / for CIDRs expressed in bits
        # We'd use a URL encoder but it's an edge case.
        return "{urlbase}/{destination}+{mask}".format(
            urlbase=self.url_prefix, destination=destination, mask=mask.replace("/", "%2f"))



# Copyright 2014,  Doug Wiegley,  A10 Networks.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import
from __future__ import unicode_literals

from acos_client import errors as acos_errors
from acos_client.v21 import base


class HealthMonitor(base.BaseV21):

    # Valid types
    ICMP = 0
    TCP = 1
    HTTP = 3
    HTTPS = 4

    def get(self, name, **kwargs):
        return self._post("slb.hm.search", {"name": name}, **kwargs)

    def _set(self, action, name, mon_type, interval, timeout, max_retries,
             method=None, url=None, expect_code=None, port=None, **kwargs):
        defs = {
            self.HTTP: {
                'protocol': 'http',
                'port': 80
            },
            self.HTTPS: {
                'protocol': 'https',
                'port': 443
            },
            self.ICMP: {
                'protocol': 'icmp',
            },
            self.TCP: {
                'protocol': 'tcp',
                'port': 80
            }
        }
        params = {
            'retry': max_retries,
            'name': name,
            'consec_pass_reqd': max_retries,
            'interval': interval,
            'timeout': timeout,
            'disable_after_down': 0,
            'type': mon_type,
        }
        if mon_type in defs:
            params[defs[mon_type]['protocol']] = {
                'url': "%s %s" % (method, url),
                'expect_code': expect_code,
            }
            n = port or defs[mon_type].get('port')
            if n:
                params[defs[mon_type]['protocol']]['port'] = n
        try:
            self._post(action, params, **kwargs)
        except acos_errors.HMMissingHttpPassive:
            # Some version of AxAPI 2.1 require this arg
            params[defs[mon_type]['protocol']]['passive'] = 0
            self._post(action, params, **kwargs)

    def create(self, name, mon_type, interval, timeout, max_retries,
               method=None, url=None, expect_code=None, port=None, **kwargs):
        self._set("slb.hm.create", name, mon_type, interval, timeout,
                  max_retries, method, url, expect_code, port, **kwargs)

    def update(self, name, mon_type, interval, timeout, max_retries,
               method=None, url=None, expect_code=None, port=None, **kwargs):
        self._set("slb.hm.update", name, mon_type, interval, timeout,
                  max_retries, method, url, expect_code, port, **kwargs)

    def delete(self, name, **kwargs):
        self._post("slb.hm.delete", {"name": name}, **kwargs)



# Copyright 2014,  Jeff Buttars,  A10 Networks.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import
from __future__ import unicode_literals

import random
import six
import time

from acos_client import errors as acos_errors
from acos_client.v30 import base


class Partition(base.BaseV30):

    def available(self):
        return self._get('/partition-available-id/oper/')

    def all(self):
        return self._get('/partition-all/oper')

    def get(self, name):
        z = self.all()
        if not z:
            raise acos_errors.NotFound()
        # There is no partition-list member if there are no partitions
        for p in z['partition-all']['oper'].get('partition-list', {}):
            if p['partition-name'] == name:
                return p
        raise acos_errors.NotFound()

    def old_get(self, name):
        return self._get('/partition/' + name)

    def exists(self, name):
        if name == 'shared':
            return True
        try:
            self.get(name)
            return True
        except acos_errors.NotFound:
            return False

    def active(self, name='shared'):
        if self.client.current_partition != name:
            self._post("/active-partition/" + name)
            self.client.current_partition = name

    def _next_available_id(self):
        a = self.available()
        if a is None:
            raise acos_errors.OutOfPartitions()

        z = a['partition-available-id']['oper']['range-list'][0]['start']
        return int(z)

    def _create(self, name, partition_id, application_type=None):
        params = {
            "partition": {
                "partition-name": name,
                "id": partition_id,
            }
        }

        if application_type:
            params['partition']['application-type'] = application_type

        self._post("/partition", params)

    def create(self, name, application_type=None):
        if name == 'shared':
            return

        if self.exists(name):
            raise acos_errors.Exists

        # For concurrency's sake, since we have to lookup the id and then
        # set it, loop if we get an exists error.
        for i in six.moves.range(1, 1000):
            try:
                self._create(name, self._next_available_id(), application_type)
                break
            except acos_errors.PartitionIdExists:
                time.sleep(0.05 + random.random() / 100)

    def delete(self, name):
        if name == 'shared':
            return

        self.client.session.close()
        try:
            self._delete("/partition/" + name)
        except acos_errors.NotFound:
            pass

        self.client.session.close()
        x = self.get(name)
        p = {
            'partition': {
                'partition-name': name,
                'id': x['partition-id']
            }
        }
        self._post("/delete/partition", p)

        self.client.session.close()



# Copyright 2014,  Doug Wiegley,  A10 Networks.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from __future__ import absolute_import
from __future__ import unicode_literals

import acos_client.v21.base as base

from acos_client.v21.vrrp_a.failover import VRRPAFailoverPolicy
from acos_client.v21.vrrp_a.interface import VRRPAInterface
from acos_client.v21.vrrp_a.vrrp_global import VRRPAGlobal


class VRRPA(base.BaseV21):
    # For status args
    @property
    def vrrpa_global(self):
        return VRRPAGlobal(self.client)

    @property
    def interface(self):
        return VRRPAInterface(self.client)

    @property
    def failover_policy(self):
        return VRRPAFailoverPolicy(self.client)



# Copyright 2014-2016 A10 Networks.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import acos_client.v30.base as base

from acos_client.v30.overlay.options import OverlayOptions
from acos_client.v30.overlay.vtep import OverlayVtep


class Overlay(base.BaseV30):
    @property
    def options(self):
        return OverlayOptions(self.client)

    @property
    def vtep(self):
        return OverlayVtep(self.client)



# Copyright 2014,  Jeff Buttars,  A10 Networks.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import
from __future__ import unicode_literals

from acos_client.v30.action import Action
from acos_client.v30 import base
from acos_client.v30.partition import Partition


class System(base.BaseV30):

    @property
    def action(self):
        return Action(self.client)

    @property
    def partition(self):
        return Partition(self.client)

    def information(self):
        return self._get("/system")

    def stats(self, name='', max_retries=None, timeout=None, **kwargs):
        return self._get("/system/" + name + '/stats',
                         max_retries=max_retries, timeout=timeout,
                         **kwargs)



# Copyright 2021,  Jeff Buttars,  A10 Networks.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import
from __future__ import unicode_literals


def acos_version_str2int(ver):
    if ver.isdigit():
        return int(ver)
    else:
        vnum = ""
        for index in range(len(ver)):
            if ver[index].isdigit():
                vnum = vnum + ver[index]
            else:
                break
        return int(vnum)


def acos_revision_parse(revision):
    rev = []
    revision_list = ['GR', 'P', 'SP']
    for tag in revision_list:
        tag_index = revision.find(tag)
        if tag_index >= 0:
            tag_len = len(tag)
            rev.append(acos_version_str2int(revision[(tag_index + tag_len):]))
        else:
            rev.append(0)

    return tuple(rev)


def acos_version(acos_version):
    major = acos_version_str2int(acos_version.split('.')[0])
    minor = acos_version_str2int(acos_version.split('.')[1])
    patch = acos_version_str2int(acos_version.split('.')[2])

    revision = ""
    prev = acos_version.find('-')
    if prev > 0:
        revision = acos_version[prev + 1:].upper()
    gr, p, sp = acos_revision_parse(revision)

    return (major, minor, patch, gr, p, sp)


def acos_version_cmp(ver1, ver2):
    vtup1 = acos_version(ver1)
    vtup2 = acos_version(ver2)
    for index in range(len(vtup1)):
        if vtup1[index] != vtup2[index]:
            return vtup1[index] - vtup2[index]
    return 0



# Copyright 2014,  Doug Wiegley,  A10 Networks.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import
from __future__ import unicode_literals

try:
    import unittest2 as unittest
except ImportError:
    import unittest

import acos_client


class TestUtils(unittest.TestCase):

    def test_version(self):
        exp = (4, 1, 1, 0, 0, 0)
        tup = acos_client.utils.acos_version("4.1.1")
        self.assertEqual(tup, exp)

        exp = (4, 1, 1, 0, 2, 0)
        tup = acos_client.utils.acos_version("4.1.1-P2")
        self.assertEqual(tup, exp)

        exp = (4, 1, 1, 0, 2, 0)
        tup = acos_client.utils.acos_version("4.1.1-p2")
        self.assertEqual(tup, exp)

        exp = (4, 1, 4, 1, 2, 0)
        tup = acos_client.utils.acos_version("4.1.4-GR1-P2")
        self.assertEqual(tup, exp)

        exp = (4, 1, 4, 1, 3, 4)
        tup = acos_client.utils.acos_version("4.1.4-gr1-p3-sp4")
        self.assertEqual(tup, exp)

        exp = (4, 1, 4, 1, 3, 4)
        tup = acos_client.utils.acos_version("4.1.4-GR1-P3-SP4")
        self.assertEqual(tup, exp)

        exp = (5, 2, 0, 0, 0, 0)
        tup = acos_client.utils.acos_version("5.2.0")
        self.assertEqual(tup, exp)

    def test_version_cmp(self):
        rt = acos_client.utils.acos_version_cmp("5.2.0", "4.2.0")
        self.assertGreater(rt, 0)

        rt = acos_client.utils.acos_version_cmp("5.3.0", "5.2.0")
        self.assertGreater(rt, 0)

        rt = acos_client.utils.acos_version_cmp("5.2.0", "5.2.0")
        self.assertEqual(rt, 0)

        rt = acos_client.utils.acos_version_cmp("5.2.1", "5.2.0")
        self.assertGreater(rt, 0)

        rt = acos_client.utils.acos_version_cmp("5.2.1-P1", "5.2.1")
        self.assertGreater(rt, 0)

        rt = acos_client.utils.acos_version_cmp("5.2.1-GR1-P1", "5.2.1-GR1")
        self.assertGreater(rt, 0)

        rt = acos_client.utils.acos_version_cmp("5.2.1-GR1-P3-SP1", "5.2.1-GR1-P3")
        self.assertGreater(rt, 0)

        # case insensitive
        rt = acos_client.utils.acos_version_cmp("5.2.1-GR1-p1", "5.2.1-GR1")
        self.assertGreater(rt, 0)

        rt = acos_client.utils.acos_version_cmp("5.2.1-GR1-P3-SP1", "5.2.1-gr1-p3-sp1")
        self.assertEqual(rt, 0)



# Copyright 2014,  Doug Wiegley,  A10 Networks.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import
from __future__ import unicode_literals

try:
    import unittest
    from unittest import mock
except ImportError:
    import mock
    import unittest2 as unittest

from acos_client.v30 import sflow


class TestSFlow(unittest.TestCase):
    def setUp(self):
        self.client = mock.MagicMock()
        self.target = sflow.SFlow(self.client)

    def test_collector_ip_create(self):
        ip_address = "127.0.0.1"
        port = 4242
        self.target.collector.ip.create(ip_address, port)
        expected = {"ip": [{"addr": ip_address, "port": int(port)}]}
        actual = self.client.http.request.call_args[0]
        self.assertTrue(expected in actual)

    def test_collector_ip_get(self):
        ip_address = "127.0.0.1"
        port = 4242

        self.target.collector.ip.get(ip_address, port)
        expected = "/axapi/v3/sflow/collector/ip/{0}+{1}".format(ip_address, port)
        actual = self.client.http.request.call_args[0]
        self.assertTrue(expected in actual)

    def test_setting_create(self):
        self.target.setting.create(None, None, None, 1)
        expected = {'setting': {
                    'counter-polling-interval': 1}}
        actual = self.client.http.request.call_args[0]
        self.assertTrue(expected in actual)

    def _test_polling_create(self, http_counter=False):
        self.target.polling.create(http_counter)
        expected = {'polling': {'http-counter': int(http_counter)}}
        actual = self.client.http.request.call_args[0]
        self.assertTrue(expected in actual)

    def test_polling_create_http_counter_negative(self):
        self._test_polling_create()

    def test_polling_create_http_counter_positive(self):
        self._test_polling_create(True)
